@@ -46,7 +46,7 @@ static int gnutls_decode_rs_value(const gnutls_datum_t *sig_value,
 /**
  * libjwt encryption/decryption function definitions
  */
-int jwt_sign_sha_hmac(jwt_t *jwt, char **out, unsigned int *len, const char *str)
+int jwt_sign_sha_hmac(jwt_t *jwt, char **out, unsigned int *len, const char *str, unsigned int str_len)
 {
 	int alg;
 
@@ -65,12 +65,12 @@ int jwt_sign_sha_hmac(jwt_t *jwt, char **out, unsigned int *len, const char *str
 	}
 
 	*len = gnutls_hmac_get_len(alg);
-	*out = malloc(*len);
+	*out = jwt_malloc(*len);
 	if (*out == NULL)
 		return ENOMEM;
 
-	if (gnutls_hmac_fast(alg, jwt->key, jwt->key_len, str, strlen(str), *out)) {
-		free(*out);
+	if (gnutls_hmac_fast(alg, jwt->key, jwt->key_len, str, str_len, *out)) {
+		jwt_freemem(*out);
 		*out = NULL;
 		return EINVAL;
 	}
@@ -78,13 +78,13 @@ int jwt_sign_sha_hmac(jwt_t *jwt, char **out, unsigned int *len, const char *str
 	return 0;
 }
 
-int jwt_verify_sha_hmac(jwt_t *jwt, const char *head, const char *sig)
+int jwt_verify_sha_hmac(jwt_t *jwt, const char *head, unsigned int head_len, const char *sig)
 {
 	char *sig_check, *buf = NULL;
 	unsigned int len;
 	int ret = EINVAL;
 
-	if (!jwt_sign_sha_hmac(jwt, &sig_check, &len, head)) {
+	if (!jwt_sign_sha_hmac(jwt, &sig_check, &len, head, head_len)) {
 		buf = alloca(len * 2);
 		jwt_Base64encode(buf, sig_check, len);
 		jwt_base64uri_encode(buf);
@@ -92,13 +92,13 @@ int jwt_verify_sha_hmac(jwt_t *jwt, const char *head, const char *sig)
 		if (!strcmp(sig, buf))
 			ret = 0;
 
-		free(sig_check);
+		jwt_freemem(sig_check);
 	}
 
 	return ret;
 }
 
-int jwt_sign_sha_pem(jwt_t *jwt, char **out, unsigned int *len, const char *str)
+int jwt_sign_sha_pem(jwt_t *jwt, char **out, unsigned int *len, const char *str, unsigned int str_len)
 {
 	/* For EC handling. */
 	int r_padding = 0, s_padding = 0, r_out_padding = 0,
@@ -113,7 +113,7 @@ int jwt_sign_sha_pem(jwt_t *jwt, char **out, unsigned int *len, const char *str)
 	};
 	gnutls_datum_t body_dat = {
 		(unsigned char *)str,
-		strlen(str)
+		str_len
 	};
 	gnutls_datum_t sig_dat, r, s;
 	int ret = 0, pk_alg;
@@ -183,7 +183,7 @@ int jwt_sign_sha_pem(jwt_t *jwt, char **out, unsigned int *len, const char *str)
 
 	/* RSA is very short. */
 	if (pk_alg == GNUTLS_PK_RSA) {
-		*out = malloc(sig_dat.size);
+		*out = jwt_malloc(sig_dat.size);
 		if (*out == NULL) {
 			ret = ENOMEM;
 			goto sign_clean_privkey;
@@ -222,7 +222,7 @@ int jwt_sign_sha_pem(jwt_t *jwt, char **out, unsigned int *len, const char *str)
 
 	out_size = adj << 1;
 
-	*out = malloc(out_size);
+	*out = jwt_malloc(out_size);
 	if (*out == NULL) {
 		ret = ENOMEM;
 		goto sign_clean_privkey;
@@ -249,14 +249,14 @@ sign_clean_key:
 	gnutls_x509_privkey_deinit(key);
 
 	if (ret && *out) {
-		free(*out);
+		jwt_freemem(*out);
 		*out = NULL;
 	}
 
 	return ret;
 }
 
-int jwt_verify_sha_pem(jwt_t *jwt, const char *head, const char *sig_b64)
+int jwt_verify_sha_pem(jwt_t *jwt, const char *head, unsigned int head_len, const char *sig_b64)
 {
 	gnutls_datum_t r, s;
 	gnutls_datum_t cert_dat = {
@@ -265,7 +265,7 @@ int jwt_verify_sha_pem(jwt_t *jwt, const char *head, const char *sig_b64)
 	};
 	gnutls_datum_t data = {
 		(unsigned char *)head,
-		strlen(head)
+		head_len
 	};
 	gnutls_datum_t sig_dat = { NULL, 0 };
 	gnutls_pubkey_t pubkey;
@@ -274,17 +274,23 @@ int jwt_verify_sha_pem(jwt_t *jwt, const char *head, const char *sig_b64)
 
 	switch (jwt->alg) {
 	case JWT_ALG_RS256:
-	case JWT_ALG_ES256:
 		alg = GNUTLS_DIG_SHA256;
 		break;
+	case JWT_ALG_ES256:
+		alg = GNUTLS_SIGN_ECDSA_SHA256;
+		break;
 	case JWT_ALG_RS384:
-	case JWT_ALG_ES384:
 		alg = GNUTLS_DIG_SHA384;
 		break;
+	case JWT_ALG_ES384:
+		alg = GNUTLS_SIGN_ECDSA_SHA384;
+		break;
 	case JWT_ALG_RS512:
-	case JWT_ALG_ES512:
 		alg = GNUTLS_DIG_SHA512;
 		break;
+	case JWT_ALG_ES512:
+		alg = GNUTLS_SIGN_ECDSA_SHA512;
+		break;
 	default:
 		return EINVAL;
 	}
@@ -353,7 +359,7 @@ verify_clean_pubkey:
 	gnutls_pubkey_deinit(pubkey);
 
 verify_clean_sig:
-	free(sig);
+	jwt_freemem(sig);
 
 	return ret;
 }
